import unittest
from mock import patch, MagicMock

from celery import signals, Celery

from cadasta.workertoolbox.conf import Config


class TestConfigFunctional(unittest.TestCase):
    """ Ensure that our configuration provides expected functionality """

    @classmethod
    @patch('kombu.transport.SQS.Channel.sqs', MagicMock())
    def setUpClass(cls):
        cls.q_name = 'export'
        cls.conf = Config(queues=(cls.q_name,))
        cls.app = Celery()
        cls.app.config_from_object(cls.conf)
        signals.worker_init.send(sender=None)
        cls.channel = cls.app.connection().channel()

    def test_default_exchange_type(self):
        """ Ensure default exchange is topic exchange """
        def_exch = self.app.conf.task_default_exchange
        exch_type = self.channel.typeof(def_exch).type
        self.assertEqual(exch_type, 'topic')

    def test_default_exchange_routing(self):
        """ Ensure default exchange routes tasks to multiple queues """
        exchange = self.app.conf.task_default_exchange
        queues = self.channel.typeof(exchange).lookup(
            self.channel.get_table(exchange),
            exchange, self.q_name, self.app.conf.task_default_queue)
        self.assertEqual(len(queues), 2)
        self.assertTrue(self.q_name in queues)
        self.assertTrue(self.conf.PLATFORM_QUEUE_NAME in queues)

    def test_celery_exchange_routing(self):
        """
        Ensure celery queue and platform queue are registered with default
        exchange
        """
        exchange = self.app.conf.task_default_exchange
        queues = self.channel.typeof(exchange).lookup(
            table=self.channel.get_table(exchange),
            exchange=exchange, routing_key='celery',
            default=self.app.conf.task_default_queue)

        self.assertEqual(len(queues), 2)
        self.assertTrue('celery' in queues)
        self.assertTrue(self.conf.PLATFORM_QUEUE_NAME in queues)

    def test_celery_task_routing(self):
        """ Ensure celery tasks route to celery queue and platform queue """
        options = self.app.amqp.router.route({}, 'celery.chord_unlock')
        self.assertNotIn('queue', options)
        self.assertIn('exchange', options)
        self.assertIn('routing_key', options)
        exchange = options['exchange'].name
        routing_key = options['routing_key']

        queues = self.channel.typeof(exchange).lookup(
            table=self.channel.get_table(exchange),
            exchange=exchange, routing_key=routing_key,
            default=self.app.conf.task_default_queue)
        self.assertEqual(len(queues), 2)
        self.assertTrue('celery' in queues)
        self.assertTrue(self.conf.PLATFORM_QUEUE_NAME in queues)



from celery.signals import worker_init


@worker_init.connect
def setup_exchanges(**kwargs):
    """ Setup result exchange to route all tasks to platform queue """
    from celery import app as _app

    app = _app.app_or_default()
    p = app.amqp.producer_pool.acquire()
    try:
        for q in app.conf.task_queues:
            p.maybe_declare(q)
    finally:
        p.release()



import codecs
import os
import re
import sys
from setuptools import setup, find_packages, Command


###
NAME = 'cadasta-workertoolbox'
META_PATH = os.path.join("cadasta", "workertoolbox", "__init__.py")
PACKAGES = find_packages()
CLASSIFIERS = [
    "Development Status :: 3 - Alpha",
]
###


def read(*parts):
    """
    Build an absolute path from *parts* and and return the contents of the
    resulting file.  Assume UTF-8 encoding.
    """
    cur_dir = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(cur_dir, *parts), "rb", "utf-8") as f:
        return f.read()


META_FILE = read(META_PATH)


def find_meta(meta):
    """
    Extract __*meta*__ from META_FILE.
    """
    meta_match = re.search(
        r"^__{meta}__ = ['\"]([^'\"]*)['\"]".format(meta=meta),
        META_FILE, re.M
    )
    if meta_match:
        return meta_match.group(1)
    raise RuntimeError("Unable to find __{meta}__ string.".format(meta=meta))


class CleanCommand(Command):
    """ Custom clean command to tidy up the project root. """
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        os.system('rm -vrf ./build ./dist ./*.pyc ./*.tgz ./*.egg-info')


class PublishCommand(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        if os.system("pip freeze | grep twine"):
            print("twine not installed.\nUse `pip install twine`.\nExiting.")
            sys.exit()
        os.system("python setup.py sdist")
        os.system("python setup.py bdist_wheel")
        os.system("twine upload dist/*")


class TagCommand(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        os.system("git tag -a {0} -m 'version {0}'".format(find_meta('version')))
        os.system("git push --tags")


if __name__ == "__main__":
    try:
        import pypandoc
        long_description = pypandoc.convert('README.md', 'rst')
    except(IOError, ImportError):
        long_description = open('README.md').read()

    setup(
        name=NAME,
        packages=PACKAGES,
        classifiers=CLASSIFIERS,

        version=find_meta('version'),
        description=find_meta('description'),
        author=find_meta('author'),
        author_email=find_meta('email'),
        license=find_meta('license'),
        url=find_meta('url'),

        long_description=long_description,

        install_requires=[
            "SQLAlchemy<=1.1.11",
            "boto3<=1.4.4",
            "celery<=4.0.2",
            "kombu<=4.0.3",
            "psycopg2<=2.7.1",
        ],
        dependency_links=[
            "git+https://github.com/celery/kombu.git@master#egg=kombu-4.0.3"
        ],

        cmdclass={
            'clean': CleanCommand,
            'publish': PublishCommand,
            'tag': TagCommand,
        }
    )
